#include "neural_net.hpp"
#include <functional>
#include <numeric>
#include <random>
#include <vector>

template <class ActivationFunction>
NeuralNet<ActivationFunction>::NeuralNet(std::vector<size_t> &layer_sizes)
    : m_sizes(layer_sizes) {
  int total_neurons = std::accumulate(layer_sizes.begin(), layer_sizes.end(), 0,
                                      std::plus<size_t>());
  // Initialize the activation function
  m_activation_func = ActivationFunction();

  // Create random sampling device
  std::random_device rd{};
  std::mt19937 gen{rd()};
  std::normal_distribution<float> dist{0.0, 1.0};

  // Initialize the weights
  m_weights.reserve(total_neurons);
  int start_idx = 0;
  for (auto size : m_sizes) {
    for (int i = 0; i < size; i++) {
      m_weights[i + start_idx] =
          dist(gen) * m_activation_func.init_stddev(size);
    }
    start_idx += size;
  }
}
